Package: factoextra
Type: Package
Title: Extract and Visualize the Results of Multivariate Data Analyses
Version: 2.1.0
Date: 2026-06-26
Authors@R: c(
    person("Alboukadel", "Kassambara", role = c("aut", "cre"), email = "alboukadel.kassambara@gmail.com",
           comment = c(ORCID = "0009-0002-9136-0791")),
    person("Fabian", "Mundt", role = "aut", email = "f.mundt@inventionate.de"),
    person("Laszlo", "Erdey", role = "ctb", email = "erdey.laszlo@econ.unideb.hu",
           comment = c(ORCID = "0000-0002-6781-4303",
                       affiliation = "Faculty of Economics and Business, University of Debrecen, Hungary",
                       contribution = "Modern compatibility fixes, tests, and maintenance updates"))
    )
Description: Provides easy-to-use functions to extract and visualize the output
    of multivariate data analyses, including 'PCA' (Principal Component Analysis),
    'CA' (Correspondence Analysis), 'MCA' (Multiple Correspondence Analysis),
    'FAMD' (Factor Analysis of Mixed Data), 'MFA' (Multiple Factor Analysis),
    and 'HMFA' (Hierarchical Multiple Factor Analysis) from different R packages.
    It also includes support for supplementary qualitative variables in
    'FactoMineR' 'FAMD' and 'MFA' workflows, hardened validation for clustering
    and dimension-reduction helper workflows, backward-compatible phylogenetic
    dendrogram layout support for current 'igraph' APIs, and 'ggplot2'-based
    data visualization.
License: GPL-2
LazyData: true
LazyDataCompression: xz
Encoding: UTF-8
Depends: R (>= 4.1.0), ggplot2 (>= 3.5.2)
Imports: cluster (>= 2.1.8.2), dendextend (>= 1.19.1), FactoMineR (>=
        2.13), ggpubr (>= 0.6.3), grid, rlang (>= 1.1.7), stats,
        ggrepel (>= 0.9.5)
Suggests: ade4, ca, igraph, MASS, knitr, mclust, rmarkdown, testthat
        (>= 3.0.0)
VignetteBuilder: knitr
URL: https://github.com/kassambara/factoextra,
        https://rpkgs.datanovia.com/factoextra/index.html
BugReports: https://github.com/kassambara/factoextra/issues
Collate: 'fviz_add.R' 'eigenvalue.R' 'utilities.R' 'as_factoextra.R'
        'cluster_utilities.R' 'decathlon2.R' 'print.factoextra.R'
        'get_hmfa.R' 'fviz_hmfa.R' 'get_mfa.R' 'fviz_mfa.R'
        'deprecated.R' 'dist.R' 'fviz_dend.R' 'hcut.R' 'get_pca.R'
        'fviz_cluster.R' 'eclust.R' 'facto_summarize.R' 'fviz.R'
        'fviz_ca.R' 'fviz_contrib.R' 'fviz_cos2.R' 'fviz_ellipses.R'
        'fviz_famd.R' 'get_mca.R' 'fviz_mca.R' 'fviz_mclust.R'
        'fviz_nbclust.R' 'fviz_pca.R' 'fviz_silhouette.R' 'get_ca.R'
        'get_clust_tendency.R' 'get_famd.R' 'hkmeans.R' 'housetasks.R'
        'multishapes.R' 'poison.R' 'zzz.R'
RoxygenNote: 7.3.3
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-06-26 13:51:00 UTC; kassambara
Author: Alboukadel Kassambara [aut, cre] (ORCID:
    <https://orcid.org/0009-0002-9136-0791>),
  Fabian Mundt [aut],
  Laszlo Erdey [ctb] (ORCID: <https://orcid.org/0000-0002-6781-4303>,
    affiliation: Faculty of Economics and Business, University of
    Debrecen, Hungary, contribution: Modern compatibility fixes, tests,
    and maintenance updates)
Maintainer: Alboukadel Kassambara <alboukadel.kassambara@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-26 14:40:08 UTC
